<?php

// Dans un controller

namespace App\Controller;

use App\Controller\AppController;
use Cake\Network\Http\Client;
use Cake\Core\Configure;
use Cake\I18n\Time;

class ProjectController extends AppController {

    public $helpers = [ 'Score'];

    public function index() {
        try {
            $http = new Client();

            $response = $http->get(COMPTOIR_SRV_API . "softwares/", $this->request->query, ['type' => 'json']);

            $softwares = $response->body('json_decode');

            $datas = (json_decode($response->body, true));
            $this->request->params['paging'] = $datas['paging'];
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }

        $this->set('softwares', $softwares->softwares);
    }

    public function request() {
        try {
            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "request/" . $this->request->query('id'), [], ['type' => 'json']);
            $softwareAndRelationship = $response->body('json_decode');

            $RelationshipsUsersOf = $softwareAndRelationship->RelationshipsUsersOf;
            $relationshipsServiceProviders = $softwareAndRelationship->relationshipsServicePorviders;
            $RelationshipsWorksWellWith = $softwareAndRelationship->RelationshipsWorksWellWith;
            $RelationshipsAlternativeTo = $softwareAndRelationship->RelationshipsAlternativeTo;
            $RelationshipsBackerOf = $softwareAndRelationship->RelationshipsBackerOf;
            $RelationshipsCreatedBy = $softwareAndRelationship->RelationshipsCreatedBy;

            $software = $softwareAndRelationship->software;
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }

        $this->set('project', $software);
        $this->set('usersOf', $RelationshipsUsersOf);
        $this->set('relationshipsServicePorviders', $relationshipsServiceProviders);
        $this->set('worksWellWith', $RelationshipsWorksWellWith);
        $this->set('alternativeTo', $RelationshipsAlternativeTo);
        $this->set('backerOf', $RelationshipsBackerOf);
        $this->set('createdBy', $RelationshipsCreatedBy);
    }

    /**
     *
     */
    public function backers() {
        try {

            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "RelationshipsSoftwaresUsers/getRelationshipsBySoftwareId/", $this->request->query, ['type' => 'json']);
            $softwares = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
            $this->set([
                $message => "Error",
                $projectSoftwareRelationship => $software
            ]);
        }
        $this->set('projectUsersRelationship', $softwares);
    }

    public function alternative() {
        try {

            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "RelationshipsSoftwares/getAlternativeToRelationshipsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);
            $softwares = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('projectSoftwareRelationship', $softwares);
    }

    public function works() {
        try {

            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "RelationshipsSoftwares/getWorkingWellRelationshipsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);
            $softwares = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('projectSoftwareRelationship', $softwares);
    }

    /**
     * Get All review for a software thanks to its id.
     */
    public function reviews() {
        try {

            $http = new Client();
            $responseReview = $http->get(COMPTOIR_SRV_API . "reviews/getReviewsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);

            $reviews = $responseReview->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('reviews', $reviews);
    }

    /**
     * Get All metrics for a software thanks to its id.
     */
    public function metrics() {
        try {

            $http = new Client();
            $responseReview = $http->get(COMPTOIR_SRV_API . "RawMetricsSoftwares/getRawMetricsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);

            $metrics = $responseReview->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('metrics', $metrics);
    }

    /**
     * Get All users for a software thanks to its id.
     */
    public function users() {
        try {

            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "RelationshipsSoftwaresUsers/getUsersOfRelationshipsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);
            $usersOf = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('projectUsersrelationship', $usersOf);
    }

    /*
     *
     * Test julie & pascal
     */

    public function creator() {
        try {
            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "RelationshipsSoftwaresUsers/getCreatorOfRelationshipsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);
            $usersOf = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('projectUsersrelationship', $CreatorOf);
    }

    public function providers() {
        try {

            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "RelationshipsSoftwaresUsers/getProvidersRelationshipsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);
            $usersOf = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('projectUsersrelationship', $usersOf);
    }

    public function screenshots() {
        try {

            $http = new Client();
            $response = $http->get(COMPTOIR_SRV_API . "screenshots/getScreenshotsBySoftwareId/" . $this->request->query('id'), [], ['type' => 'json']);
            $screenshots = $response->body('json_decode');
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        $this->set('screenshotsSoftwares', $screenshots);
    }

    /**
     * Sm means SourceMetric, objective sources of metric,
     * Cm means ComptoirMetric, metrics from Comptoir du libre .
     */
    public function formula() {
        $this->set('formula', ["metrics" => ["Sm_last_commit_age" => __d("ProjectController", "The age(in days) of the last commit for the project."),
                "Sm_project_age" => __d("ProjectController", "The age of the project (in days)"),
                "Sm_delta_commit_one_month" => __d("ProjectController", "The difference between the number of commits in a month and the average of the number of commits in one month for all the projects."),
                "Sm_delta_commit_twelve_month" => __d("ProjectController", "The difference between the number of commits in twelve months and the average of the number of commits in twelve months for all the projects."),
                "Sm_number_of_contributors" => __d("ProjectController", "The number of contributors working on the project."),
                "Sm_high_committer_percent" => __d("ProjectController", "The contribution percentage of the highest contributor."),
                "Cm_code_gouv_label" => __d("ProjectController", "The project obtains the label code gov if it is available on code.gouv.fr"),
                "Cm_declared_users" => __d("ProjectController", "The number of declared users on Comptoir du libre."),
                "Cm_average_review_score" => __d("ProjectController", "The average review score for this project on Comptoir du libre."),
                "Cm_screenshots" => __d("ProjectController", "The number of screenshots available on Comptoir du libre.")
            ],
            "nbSource" => [ sizeof(Configure::read("SOURCE_CODE_METRICS"))]
        ]);
    }

}
