<?php

namespace App\View\Helper;

use Cake\Core\Configure;
use Cake\View\Helper;
use Cake\View\StringTemplateTrait;
use Cake\I18n\I18n;

class ListsHelper extends Helper
{

    use StringTemplateTrait;

    protected $_defaultConfig = [
        'templates' => [
            'review' => '<div {{reviewDiv}}><ol {{reviewOl}}><li {{reviewLi}}> <h3>{{title}}</h3></li><li>{{created}}</li><li><div {{starsDiv}}>{{stars}}</div></li><li>{{userName}}</li></ol><p{{reviewComment}}>{{comment}}</p></div>',
            'stars' => '<span{{attrGlyph}}><span{{attrStar}}>{{content}}</span></span>',
            'software'=>
                '<li>
                    <div {{attrsColumn}}>
                        <h3>{{softwareName}}</h3>
                        {{LinkImage}}
                        <p {{attrsDescription}}>{{softwareDescription}}</p>
                    </div>
                </li>',
            'user'=>'<li>
                    <div {{attrsColumn}}>
                        <h3 {{attrsTitle}}>{{name}}</h3>
                        {{LinkImage}}
                    </div>
                </li>',
            'header' => '<div class="align"><h2>{{sectionTitle}}</h2><span {{attribs}}>{{link}}</span></div>',
            'footer' => '<div class="row"><div class="col-xs-12 col-sm-12 col-md-12 col-lg-12 panda">{{link}}</div></div>'
        ]
    ];

    public $helpers = ['Html', 'Text','Time'];



    public function softwares($softwares, $limit = false)
    {
        $result = null;
        if (!empty($softwares)) {
            foreach ($softwares as $software) {
                if (empty ($software->logo_directory) || empty($software->photo)) {
                    $logo = $this->Html->link($this->Html->image("logos/Softwarelogo_placeholder.jpg",
                        ["alt" => $software->softwarename, "class" => "img-responsive size-logo hidden-xs"]),
                        ['controller' => 'Softwares', 'action' => 'view', $software->id],
                        ['escape' => false, "title" => $software->softwarename]);
                } else {
                    $logo = $this->Html->link($this->Html->image(COMPTOIR_SRV_URL . $software->logo_directory . DS . $software->photo,
                        ["alt" => $software->softwarename, "class" => "img-responsive size-logo hidden-xs"]),
                        ['controller' => 'Softwares', 'action' => 'view', $software->id],
                        ['escape' => false, "title" => $software->softwarename]);
                }

                $columnStrucuture = ($limit == false) ?
                    $this->templater()->formatAttributes(['class' => 'software-unit col-xs-6 col-sm-4 col-md-3 col-lg-2 backgroundUnit list-unstyled'])
                    : $this->templater()->formatAttributes(['class' => 'software-unit col-xs-6 col-sm-4 col-md-3 col-lg-3 backgroundUnit list-unstyled']);


                $result .= $this->formatTemplate('software',[
                    'attrsColumn' => $columnStrucuture,
                    'softwareName' =>
                        $this->Html->link(
                            $this->Text->truncate($software->softwarename, 12, ['ellipsis' => '...', 'exact' => false]),
                            ['controller' => 'Softwares', 'action' => 'view', $software->id],
                            ['escape' => false]
                        ),
                    'LinkImage' => $logo,
                    'attrsDescription' => $this->templater()->formatAttributes(['class' => 'text-overflow project-description']),
                    'softwareDescription' =>  $this->Text->truncate($software->description, 100, ['ellipsis' => '...', 'exact' => false]),
                    ['class' => 'text-overflow project-description'],
                ]);
            }
        }

        return null !== $result ? $this->Html->tag('ul', $result, ['class' => 'list-unstyled']) : null;
    }

    /**
     * @param $softwares Should be contains a list of software
     * @param array $options options for create a link and a specific message
     * @return string Returns a block of software's summary
     */
    public function softwareBlock($softwares, array $options, $limit = true)
    {
        $link = "";
        $softwareDisplay = "";
        if (!empty($softwares)) {

            if (( count($softwares) > Configure::read("MAX_DISPLAY") ) && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' =>  $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $softwareDisplay = ($limit == true) ? $this->softwares(array_slice($softwares, 0, Configure::read("MAX_DISPLAY")), $limit) : $this->softwares($softwares,$limit);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle'=> $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($softwareDisplay, '<li') == false) {
            $softwareDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );

        return $header.$softwareDisplay.$footer;
    }

    /**
     * @param $users
     * @return null
     */
    public function users($users,$limit = false)
    {
        $result = null;

        if (!empty($users)) {
            foreach ($users as $user) {
                if ((empty ($user->logo_directory) || empty($user->photo))) {
                    $logo = $this->Html->link($this->Html->image("logos/User_placeholder.jpg",
                        ["alt" => $user->username, "class" => "img-responsive size-logo"]),
                        ['controller' => 'Users', 'action' => 'view', $user->id],
                        ['escape' => false]);
                } else {
                    $logo = $this->Html->link($this->Html->image(COMPTOIR_SRV_URL . $user->logo_directory . DS . $user->photo,
                        ["alt" => $user->username, "class" => "img-responsive size-logo"]),
                        ['controller' => 'Users', 'action' => 'view', $user->id],
                        ['escape' => false]);
                }

                $columnStrucuture = ($limit == false) ?
                    $this->templater()->formatAttributes(['class' => 'software-unit col-xs-6 col-sm-4 col-md-3 col-lg-2 backgroundUnit list-unstyled'])
                    : $this->templater()->formatAttributes(['class' => 'software-unit col-xs-6 col-sm-4 col-md-3 col-lg-3 backgroundUnit list-unstyled']);


                $result .= $this->formatTemplate('user',[
                    'attrsColumn' => $columnStrucuture,
                    'name' =>
                        $this->Html->link($this->Text->truncate($user->username, 35, ['ellipsis' => '...', 'exact' => false]),
                            ['controller' => 'Users', 'action' => 'view', $user->id],
                            ['escape' => false, "title" => $user->username]
                        ),
                    'attrsTitle' => $this->templater()->formatAttributes(['class' => 'size-title' ]),
                    'LinkImage' => $logo,
                ]);
            }
        }

        return null !== $result ? $this->Html->tag('ul', $result, ['class' => 'list-unstyled']) : null;
    }

    /**
     * @param $users Should be contains a list of users
     * @param array $options options for create a link and a specific message
     * @return string Returns a block of software's summary
     */
    public function userBlock($users, array $options, $limit=true)
    {
        $link = "";
        $userDisplay = "";
        if (!empty($users)) {

            if ( (count($users) > Configure::read("MAX_DISPLAY") )&& $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' =>  $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $userDisplay = ($limit == true) ? $this->users(array_slice($users, 0, Configure::read("MAX_DISPLAY")),$limit) : $this->users($users,$limit);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle'=> $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($userDisplay, '<li') == false) {
            $userDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );

        return $header.$userDisplay.$footer;
    }

    /**
     * @param $screenshots
     * @return null
     */
    public function screenshots($screenshots)
    {
        $result = null;
        if (!empty($screenshots)) {
            foreach ($screenshots as $screenshot) {
                if (!empty ($screenshot->url_directory) || !empty($screenshot->photo)) {
                    $result .= $this->Html->tag(
                        'li',
                        $this->Html->tag(
                            'div',
                            $this->Html->link($this->Html->image(COMPTOIR_SRV_URL . $screenshot->url_directory . DS . $screenshot->photo,
                                ['alt' => __d("Screenshot of software {0}", ""), 'class' => 'img-responsive ']),
                                COMPTOIR_SRV_URL . $screenshot->url_directory . DS . $screenshot->photo,
                                ['escape' => false])
                            ,
                            ['class' => 'size-title"']
                        )
                        ,
                        ["class" => " col-xs-12 col-sm-6 col-md-3 col-lg-3 screenShot"]
                    );
                }
            }
        }

        return null !== $result ? $this->Html->tag(
          'div',
            $this->Html->tag('ul', $result, ['class' => 'list-unstyled']),
            ["class"=>"row"]
        ): null;

    }


    public function screenshotBlock($items, array $options, $limit = true)
    {
        $link = "";
        $screenshotDisplay = "";
        if (!empty($items)) {

            if ( ( count($items) > Configure::read("MAX_DISPLAY") ) && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' =>  $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $screenshotDisplay = ($limit == true) ? $this->screenshots(array_slice($items, 0, Configure::read("MAX_DISPLAY"))) : $this->screenshots($items);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle'=> $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($screenshotDisplay, '<li') == false) {
            $screenshotDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );

        return $header.$screenshotDisplay.$footer;
    }


    public function reviews($reviews)
    {
        $result = "";
        if (!empty($reviews)) {
            foreach ($reviews as $review) {

                $starsElement = "";
                for ($stars = 0; $stars < $review->evaluation; $stars ++){
                    $starsElement.= $this->formatTemplate('stars',
                        [
                            'attrGlyph' => $this->templater()->formatAttributes(["class"=>"glyphicon glyphicon-star"]),
                            'attrStar' => $this->templater()->formatAttributes(["class"=>"sr-only"]),
                            'content' => "*",
                        ]
                    );
                }

                $result.= $this->formatTemplate('review',[
                    'reviewDiv' => $this->templater()->formatAttributes(['class' => "col-xs-11 col-sm-6 col-md-6 col-lg-6 backgroundReview" ]),
                    'reviewOl' => $this->templater()->formatAttributes(['class'=> "list-unstyled list-inline" ]),
                    'reviewLi' => $this->templater()->formatAttributes(['class' => "col-xs-11 col-sm-11 col-md-11 col-lg-11" ]),
                    'title'=> $this->Text->truncate($review->title,100,['ellipsis' => '...','exact' => false]),
                    'userName' => $this->Html->link($this->Text->truncate($review->user->username, 35, ['ellipsis' => '...', 'exact' => false]),
                        ['controller' => 'Users', 'action' => 'view', $review->user->id],
                        ['escape' => false, "title" => $review->user->username]),
                    'created' => $this->Time->format(strtotime($review->created),
                                    [\IntlDateFormatter::SHORT,-1],
                                    I18n::locale()
                                ),
                    'starDiv'=> $this->templater()->formatAttributes(['class'=>'ratingStar']),
                    'stars'=>$starsElement.$review->evaluation,
                    'reviewComment'=> $this->templater()->formatAttributes(['class' => "col-xs-11 col-sm-11 col-md-11 col-lg-11" ]),
                    'comment'=> $this->Text->truncate($review->comment,350,['ellipsis' => '...','exact' => false]),

                ]);

            }
        }

        return $result;
    }


    public function reviewBlock($items ,array $options, $limit = true)
    {
        $link = "";
        $reviewDisplay = "";
        if (!empty($items)) {

            if (count($items) > Configure::read("MAX_DISPLAY") && $this->request->action == 'view') {

                $link = $this->Html->link($options["tooManyMsg"],
                    ['controller' => $this->request->controller, 'action' => $options["link"]["action"], $options["link"]["id"]],
                    ['title' =>  $options["titleSeeAll"]],
                    ['escape' => false]);
            }

            $reviewDisplay = ($limit == true) ? $this->reviews(array_slice($items, 0, Configure::read("MAX_DISPLAY"))) : $this->reviews($items);
        }

        $header = $this->formatTemplate(
            'header',
            [
                'sectionTitle'=> $options["title"],
                'spanAttr' => $this->templater()->formatAttributes($options),
                'link' => $link
            ]
        );

        if (strpos($reviewDisplay, '<li') == false) {
            $reviewDisplay .= $this->Html->tag(
                'p',
                $options["emptyMsg"]
            );
        }

        $participateLink = $this->Html->link($options['link']["participate"],
            ['controller' => 'Home', 'action' => 'participate'],
            ['escape' => false]);

        $footer = $this->formatTemplate(
            'footer',
            [
                'link' => $participateLink
            ]
        );

        return $header.$reviewDisplay.$footer;
    }

}
